@@ -253,15 +253,32 @@
 /* SECURITY
 /* .ad
 /* .fi
-/*	By design, this program is not set-user (or group) id. However,
-/*	it must handle data from untrusted, possibly remote, users.
-/*	Thus, the usual precautions need to be taken against malicious
-/*	inputs.
+/*	By design, this program is not set-user (or group) id.
+/*	It is prepared to handle message content from untrusted,
+/*	possibly remote, users.
 /*
-/*	In particular, applications that invoke the Postfix
-/*	\fBsendmail\fR(1) command MUST disable command options processing
-/*	for all command arguments that contain user-specified data,
-/*	by structuring the command line as follows:
+/*	However, like most Postfix programs, this program does not
+/*	enforce a security policy on its command-line arguments.
+/*	Instead, it relies on the UNIX system to enforce access
+/*	policies based on the effective user and group IDs of the
+/*	process. Concretely, this means that running Postfix commands
+/*	as root (from sudo or equivalent) on behalf of a non-root
+/*	user is likely to create privilege escalation opportunities.
+/*
+/*	If an application runs any Postfix programs on behalf of
+/*	users that do not have normal shell access to Postfix
+/*	commands, then that application MUST restrict user-specified
+/*	command-line arguments to avoid privilege escalation.
+/* .IP \(bu
+/*	Filter all command-line arguments, for example arguments
+/*	that contain a pathname or that specify a database access
+/*	method. These pathname checks must reject user-controlled
+/*	symlinks or hardlinks to sensitive files, and must not be
+/*	vulnerable to TOCTOU race attacks.
+/* .IP \(bu
+/*	Disable command options processing for all command arguments
+/*	that contain user-specified data. For example, the Postfix
+/*	\fBsendmail\fR(1) command line MUST be structured as follows:
 /*
 /* .nf
 /*	    \fB/path/to/sendmail\fR \fIsystem-arguments\fR \fB--\fR \fIuser-arguments\fR
@@ -269,7 +286,7 @@
 /*
 /*	Here, the "\fB--\fR" disables command option processing for
 /*	all \fIuser-arguments\fR that follow.
-/*
+/* .IP
 /*	Without the "\fB--\fR", a malicious user could enable Postfix
 /*	\fBsendmail\fR(1) command options, by specifying an email
 /*	address that starts with "\fB-\fR".
@@ -386,7 +403,7 @@
 /*	The time after which the sender receives a copy of the message
 /*	headers of mail that is still queued.
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
-/*	The list of environment parameters that a privileged Postfix
+/*	The list of environment variables that a privileged Postfix
 /*	process will import from a non-Postfix parent process, or name=value
 /*	environment overrides.
 /* .IP "\fBmail_owner (postfix)\fR"
@@ -677,6 +694,7 @@ static void enqueue(const int flags, const char *encoding,
     VSTRING *postdrop_command;
     uid_t   uid = getuid();
     int     status;
+    VSTRING *why;			/* postdrop status message */
     int     naddr;
     int     prev_type;
     MIME_STATE *mime_state = 0;
@@ -970,11 +988,15 @@ static void enqueue(const int flags, const char *encoding,
     if (vstream_ferror(VSTREAM_IN))
 	msg_fatal_status(EX_DATAERR, "%s(%ld): error reading input: %m",
 			 saved_sender, (long) uid);
-    if ((status = mail_stream_finish(handle, (VSTRING *) 0)) != 0)
+    why = vstring_alloc(100);
+    if ((status = mail_stream_finish(handle, why)) != CLEANUP_STAT_OK)
 	msg_fatal_status((status & CLEANUP_STAT_BAD) ? EX_SOFTWARE :
 			 (status & CLEANUP_STAT_WRITE) ? EX_TEMPFAIL :
+			 (status & CLEANUP_STAT_NOPERM) ? EX_NOPERM :
 			 EX_UNAVAILABLE, "%s(%ld): %s", saved_sender,
-			 (long) uid, cleanup_strerror(status));
+			 (long) uid, VSTRING_LEN(why) ?
+			 STR(why) : cleanup_strerror(status));
+    vstring_free(why);
 
     /*
      * Don't leave them in the dark.
@@ -1030,6 +1052,7 @@ int     main(int argc, char **argv)
     const char *dsn_envid = 0;
     int     saved_optind;
     ARGV   *import_env;
+    char   *alias_map_from_args = 0;
 
     /*
      * Fingerprint executables and core dumps.
@@ -1299,9 +1322,7 @@ int     main(int argc, char **argv)
 	    case 'A':
 		if (optarg[1] == 0)
 		    msg_fatal_status(EX_USAGE, "-oA requires pathname");
-		myfree(var_alias_db_map);
-		var_alias_db_map = mystrdup(optarg + 1);
-		set_mail_conf_str(VAR_ALIAS_DB_MAP, var_alias_db_map);
+		alias_map_from_args = optarg + 1;
 		break;
 	    case '7':
 	    case '8':
@@ -1451,14 +1472,17 @@ int     main(int argc, char **argv)
 	if (argv[OPTIND])
 	    msg_fatal_status(EX_USAGE,
 			 "alias initialization mode requires no recipient");
-	if (*var_alias_db_map == 0)
+	if (alias_map_from_args == 0 && *var_alias_db_map == 0)
 	    return (0);
-	ext_argv = argv_alloc(2);
+	ext_argv = argv_alloc(3);
 	argv_add(ext_argv, "postalias", (char *) 0);
 	for (n = 0; n < msg_verbose; n++)
 	    argv_add(ext_argv, "-v", (char *) 0);
 	argv_add(ext_argv, "--", (char *) 0);
-	argv_split_append(ext_argv, var_alias_db_map, CHARS_COMMA_SP);
+	if (alias_map_from_args != 0)
+	    argv_add(ext_argv, alias_map_from_args, (char *) 0);
+	else
+	    argv_split_append(ext_argv, var_alias_db_map, CHARS_COMMA_SP);
 	argv_terminate(ext_argv);
 	mail_run_replace(var_command_dir, ext_argv->argv);
 	/* NOTREACHED */
